#!/bin/bash

set -e

COMPOSE_PATH="/etc/docker/compose"
CUSTOM_ENV_FILE="custom.env"
MM_DB_SCRIPT_PATH="/var/lib/xivo-chat-backend/scripts/"
MM_DB_CREATION_SCRIPT="mattermost-create-db.sh"

set_custom_env_parameter() {
    local PARAMETER_NAME=$1
    local PARAMETER_VALUE=$2
    sed -i "/^${PARAMETER_NAME}.*$/d" ${COMPOSE_PATH}/${CUSTOM_ENV_FILE}
    echo "${PARAMETER_NAME}"="${PARAMETER_VALUE}" >> ${COMPOSE_PATH}/${CUSTOM_ENV_FILE}
}

create_mmuser_db_password() {
    MMUSER_DB_PASSWORD=$(< /dev/urandom tr -dc A-Za-z0-9 | head -c11;)
}

create_mmadmin_password() {
    MMADMIN_PASSWORD=$(< /dev/urandom tr -dc A-Za-z0-9 | head -c11;)
}

set_mmuser_db_secret_custom_env_file() {
    create_mmuser_db_password
    create_mmadmin_password
    set_custom_env_parameter "MMUSER_DB_PASSWORD" "${MMUSER_DB_PASSWORD}"
    set_custom_env_parameter "MMADMIN_PASSWORD" "${MMADMIN_PASSWORD}"
}

setup_mm_environment() {
    if [ ! "$(getent group mattermost)" ]; then
        addgroup --gid 2000 mattermost
    fi

    if [ ! "$(getent passwd mattermost)" ]; then
        adduser --system --uid 2000 --gid 2000 --disabled-password --disabled-login --no-create-home mattermost
    fi

    MATTERMOST_BASE_DIR=/var/lib/mattermost
    mkdir -p ${MATTERMOST_BASE_DIR}
    mkdir -pv ${MATTERMOST_BASE_DIR}/{data,logs,config,plugins,client-plugins}
    chown -R 2000:2000 ${MATTERMOST_BASE_DIR}
}

ask_restart() {
    if ( ! whiptail --title "Restarting XiVO CC Services." --yesno "XiVO CC Services will be restarted. Would you like to continue?" 8 78); then
        echo "Please re-launch the upgrade when ready."
        exit 100
    fi
}

recreate_pgxivocc() {
  xivocc-dcomp stop
  xivocc-dcomp up -d pgxivocc
}

create_mm_db_in_pgxivocc() {
  # Get pgxivocc container id
  local pgxivocc_container
  pgxivocc_container=$(xivocc-dcomp ps -q pgxivocc)
  # Copy script in container
  docker cp "${MM_DB_SCRIPT_PATH}/${MM_DB_CREATION_SCRIPT}" "${pgxivocc_container}":/var/tmp/
  if xivocc-dcomp exec pgxivocc bash /var/tmp/${MM_DB_CREATION_SCRIPT}; then
    echo "Mattermost database creation succeeded."
  else
    echo "Mattermost database creation failed."
    exit 100
  fi
}

pull_and_start_mm() {
    xivocc-dcomp pull
    xivocc-dcomp up -d
}

is_mattermost_running() {
    xivocc-dcomp exec mattermost mattermost-cli team list > /dev/null
    [ $? -eq 0 ]
}


waiting_for_mattermost() {
  local wait_time=60
  echo -n "Waiting $wait_time seconds for mattermost to start..."

  local mm_running="false"
  for _ in $(seq 1 $wait_time); do
      if is_mattermost_running; then
          mm_running="true"
          break
      else
          sleep 1
          echo -n "."
      fi
  done
  echo
  if [ "$mm_running" = "false" ]; then
      echo "ERROR: Mattermost is not running."
      exit 1
  fi
}

configure_mm() {
    xivocc-dcomp exec mattermost mattermost-cli team create --name xivo --display_name "XiVO" > /dev/null
    echo -e "\e[1;33m...team created\e[0m"
    xivocc-dcomp exec mattermost mattermost-cli user create --system_admin --username admin --password "${MMADMIN_PASSWORD}" --email root@localhost > /dev/null
    echo -e "\e[1;33m...admin user created\e[0m"
    xivocc-dcomp exec mattermost mattermost-cli team add xivo admin > /dev/null
    echo -e "\e[1;33m...admin user added to team\e[0m"
}

create_admin_token() {
    local ADMIN_TOKEN
    ADMIN_TOKEN=$(xivocc-dcomp exec mattermost create-admin-token)
    local EXIT_CODE=$?
    if [ $EXIT_CODE -eq 0 ]; then
        set_custom_env_parameter "CHAT_SERVER_ADMIN_TOKEN" "$ADMIN_TOKEN"
    else
        echo "Error while creating admin token, command output follows"
        echo "> ${ADMIN_TOKEN}"
        exit $EXIT_CODE
    fi
}

enable_xuc_integration() {
    set_custom_env_parameter "CHAT_SERVER_ENABLE" "true"
}

restart_services_if_needed() {
    xivocc-dcomp up -d
}

main() {
    echo -e "\e[1;33mMattermost chat backend configuration...\e[0m"
    ask_restart
    set_mmuser_db_secret_custom_env_file
    setup_mm_environment
    recreate_pgxivocc
    echo -e "\e[1;33m...creating database\e[0m"
    create_mm_db_in_pgxivocc
    echo -e "\e[1;33m...starting mattermost\e[0m"
    pull_and_start_mm
    echo -e "\e[1;33m...waiting for mattermost to be ready\e[0m"
    waiting_for_mattermost
    echo -e "\e[1;33m...creating mattermost base config\e[0m"
    configure_mm
    echo -e "\e[1;33m...creating admin token\e[0m"
    create_admin_token
    echo -e "\e[1;33m...enabling link with xuc\e[0m"
    enable_xuc_integration
    restart_services_if_needed

    echo -e "\e[1;33mMattermost chat backend installed and ready to be used.\e[0m"
}

main "${@}"

exit 0
